@@ -1,11 +1,14 @@
+import re
+import logging
+
 from .models import Model, BufferModel
 from .fields import DateField, StringField
 from .engines import MergeTree
 from .utils import get_subclass_names
 
-import logging
+# pylint: disable=R0903
 
-logger = logging.getLogger('migrations')
+logger = logging.getLogger("migrations")
 
 
 class Operation:
@@ -14,7 +17,7 @@ class Operation:
     """
 
     def apply(self, database):
-        raise NotImplementedError()   # pragma: no cover
+        raise NotImplementedError()  # pragma: no cover
 
 
 class ModelOperation(Operation):
@@ -30,9 +33,9 @@ class ModelOperation(Operation):
         self.table_name = model_class.table_name()
 
     def _alter_table(self, database, cmd):
-        '''
+        """
         Utility for running ALTER TABLE commands.
-        '''
+        """
         cmd = "ALTER TABLE $db.`%s` %s" % (self.table_name, cmd)
         logger.debug(cmd)
         database.raw(cmd)
@@ -44,7 +47,7 @@ class CreateTable(ModelOperation):
     """
 
     def apply(self, database):
-        logger.info('    Create table %s', self.table_name)
+        logger.info("    Create table %s", self.table_name)
         if issubclass(self.model_class, BufferModel):
             database.create_table(self.model_class.engine.main_model)
         database.create_table(self.model_class)
@@ -65,7 +68,7 @@ class AlterTable(ModelOperation):
         return [(row.name, row.type) for row in database.select(query)]
 
     def apply(self, database):
-        logger.info('    Alter table %s', self.table_name)
+        logger.info("    Alter table %s", self.table_name)
 
         # Note that MATERIALIZED and ALIAS fields are always at the end of the DESC,
         # ADD COLUMN ... AFTER doesn't affect it
@@ -74,8 +77,8 @@ class AlterTable(ModelOperation):
         # Identify fields that were deleted from the model
         deleted_fields = set(table_fields.keys()) - set(self.model_class.fields())
         for name in deleted_fields:
-            logger.info('        Drop column %s', name)
-            self._alter_table(database, 'DROP COLUMN %s' % name)
+            logger.info("        Drop column %s", name)
+            self._alter_table(database, "DROP COLUMN %s" % name)
             del table_fields[name]
 
         # Identify fields that were added to the model
@@ -83,34 +86,44 @@ class AlterTable(ModelOperation):
         for name, field in self.model_class.fields().items():
             is_regular_field = not (field.materialized or field.alias)
             if name not in table_fields:
-                logger.info('        Add column %s', name)
-                cmd = 'ADD COLUMN %s %s' % (name, field.get_sql(db=database))
+                logger.info("        Add column %s", name)
+                cmd = "ADD COLUMN %s %s" % (name, field.get_sql(db=database))
                 if is_regular_field:
                     if prev_name:
-                        cmd += ' AFTER %s' % prev_name
+                        cmd += " AFTER %s" % prev_name
                     else:
-                        cmd += ' FIRST'
+                        cmd += " FIRST"
                 self._alter_table(database, cmd)
 
             if is_regular_field:
-                # ALIAS and MATERIALIZED fields are not stored in the database, and raise DatabaseError
+                # ALIAS and MATERIALIZED fields are not stored in the database
+                # and raise DatabaseError
                 # (no AFTER column). So we will skip them
                 prev_name = name
 
         # Identify fields whose type was changed
         # The order of class attributes can be changed any time, so we can't count on it
-        # Secondly, MATERIALIZED and ALIAS fields are always at the end of the DESC, so we can't expect them to save
+        # Secondly, MATERIALIZED and ALIAS fields are always at the end of the DESC
+        # so we can't expect them to save
         # attribute position. Watch https://github.com/Infinidat/infi.clickhouse_orm/issues/47
-        model_fields = {name: field.get_sql(with_default_expression=False, db=database)
-                        for name, field in self.model_class.fields().items()}
+        model_fields = {
+            name: field.get_sql(with_default_expression=False, db=database)
+            for name, field in self.model_class.fields().items()
+        }
         for field_name, field_sql in self._get_table_fields(database):
             # All fields must have been created and dropped by this moment
-            assert field_name in model_fields, 'Model fields and table columns in disagreement'
+            assert field_name in model_fields, "Model fields and table columns in disagreement"
 
             if field_sql != model_fields[field_name]:
-                logger.info('        Change type of column %s from %s to %s', field_name, field_sql,
-                            model_fields[field_name])
-                self._alter_table(database, 'MODIFY COLUMN %s %s' % (field_name, model_fields[field_name]))
+                logger.info(
+                    "        Change type of column %s from %s to %s",
+                    field_name,
+                    field_sql,
+                    model_fields[field_name],
+                )
+                self._alter_table(
+                    database, "MODIFY COLUMN %s %s" % (field_name, model_fields[field_name])
+                )
 
 
 class AlterTableWithBuffer(ModelOperation):
@@ -135,7 +148,7 @@ class DropTable(ModelOperation):
     """
 
     def apply(self, database):
-        logger.info('    Drop table %s', self.table_name)
+        logger.info("    Drop table %s", self.table_name)
         database.drop_table(self.model_class)
 
 
@@ -148,28 +161,27 @@ class AlterConstraints(ModelOperation):
     """
 
     def apply(self, database):
-        logger.info('    Alter constraints for %s', self.table_name)
+        logger.info("    Alter constraints for %s", self.table_name)
         existing = self._get_constraint_names(database)
         # Go over constraints in the model
-        for constraint in self.model_class._constraints.values():
+        for constraint in self.model_class._constraints.values():  # pylint: disable=W0212
             # Check if it's a new constraint
             if constraint.name not in existing:
-                logger.info('        Add constraint %s', constraint.name)
-                self._alter_table(database, 'ADD %s' % constraint.create_table_sql())
+                logger.info("        Add constraint %s", constraint.name)
+                self._alter_table(database, "ADD %s" % constraint.create_table_sql())
             else:
                 existing.remove(constraint.name)
         # Remaining constraints in `existing` are obsolete
         for name in existing:
-            logger.info('        Drop constraint %s', name)
-            self._alter_table(database, 'DROP CONSTRAINT `%s`' % name)
+            logger.info("        Drop constraint %s", name)
+            self._alter_table(database, "DROP CONSTRAINT `%s`" % name)
 
     def _get_constraint_names(self, database):
         """
         Returns a set containing the names of existing constraints in the table.
         """
-        import re
-        table_def = database.raw('SHOW CREATE TABLE $db.`%s`' % self.table_name)
-        matches = re.findall(r'\sCONSTRAINT\s+`?(.+?)`?\s+CHECK\s', table_def)
+        table_def = database.raw("SHOW CREATE TABLE $db.`%s`" % self.table_name)
+        matches = re.findall(r"\sCONSTRAINT\s+`?(.+?)`?\s+CHECK\s", table_def)
         return set(matches)
 
 
@@ -191,33 +203,32 @@ class AlterIndexes(ModelOperation):
         self.reindex = reindex
 
     def apply(self, database):
-        logger.info('    Alter indexes for %s', self.table_name)
+        logger.info("    Alter indexes for %s", self.table_name)
         existing = self._get_index_names(database)
         logger.info(existing)
         # Go over indexes in the model
-        for index in self.model_class._indexes.values():
+        for index in self.model_class._indexes.values():  # pylint: disable=W0212
             # Check if it's a new index
             if index.name not in existing:
-                logger.info('        Add index %s', index.name)
-                self._alter_table(database, 'ADD %s' % index.create_table_sql())
+                logger.info("        Add index %s", index.name)
+                self._alter_table(database, "ADD %s" % index.create_table_sql())
             else:
                 existing.remove(index.name)
         # Remaining indexes in `existing` are obsolete
         for name in existing:
-            logger.info('        Drop index %s', name)
-            self._alter_table(database, 'DROP INDEX `%s`' % name)
+            logger.info("        Drop index %s", name)
+            self._alter_table(database, "DROP INDEX `%s`" % name)
         # Reindex
         if self.reindex:
-            logger.info('        Build indexes on table')
-            database.raw('OPTIMIZE TABLE $db.`%s` FINAL' % self.table_name)
+            logger.info("        Build indexes on table")
+            database.raw("OPTIMIZE TABLE $db.`%s` FINAL" % self.table_name)
 
     def _get_index_names(self, database):
         """
         Returns a set containing the names of existing indexes in the table.
         """
-        import re
-        table_def = database.raw('SHOW CREATE TABLE $db.`%s`' % self.table_name)
-        matches = re.findall(r'\sINDEX\s+`?(.+?)`?\s+', table_def)
+        table_def = database.raw("SHOW CREATE TABLE $db.`%s`" % self.table_name)
+        matches = re.findall(r"\sINDEX\s+`?(.+?)`?\s+", table_def)
         return set(matches)
 
 
@@ -225,16 +236,17 @@ class RunPython(Operation):
     """
     A migration operation that executes a Python function.
     """
+
     def __init__(self, func):
-        '''
+        """
         Initializer. The given Python function will be called with a single
         argument - the Database instance to apply the migration to.
-        '''
+        """
         assert callable(func), "'func' argument must be function"
         self._func = func
 
     def apply(self, database):
-        logger.info('    Executing python operation %s', self._func.__name__)
+        logger.info("    Executing python operation %s", self._func.__name__)
         self._func(database)
 
 
@@ -244,17 +256,17 @@ class RunSQL(Operation):
     """
 
     def __init__(self, sql):
-        '''
+        """
         Initializer. The given sql argument must be a valid SQL statement or
         list of statements.
-        '''
+        """
         if isinstance(sql, str):
             sql = [sql]
         assert isinstance(sql, list), "'sql' argument must be string or list of strings"
         self._sql = sql
 
     def apply(self, database):
-        logger.info('    Executing raw SQL operations')
+        logger.info("    Executing raw SQL operations")
         for item in self._sql:
             database.raw(item)
 
@@ -268,11 +280,11 @@ class MigrationHistory(Model):
     module_name = StringField()
     applied = DateField()
 
-    engine = MergeTree('applied', ('package_name', 'module_name'))
+    engine = MergeTree("applied", ("package_name", "module_name"))
 
     @classmethod
     def table_name(cls):
-        return 'infi_clickhouse_orm_migrations'
+        return "infi_clickhouse_orm_migrations"
 
 
 # Expose only relevant classes in import *
